# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


from __future__ import absolute_import

import unittest

import LAMP


class TestSensorEvent(unittest.TestCase):
    """SensorEvent unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testSensorEvent(self):
        """Test SensorEvent"""
        # FIXME: construct object with mandatory attributes with example values
        # model = LAMP.SensorEvent()
        pass


if __name__ == '__main__':
    unittest.main()
